function u=SplitBregDebP(g,ker,blur,mu,lambda,tol,frame,Level,wLevel,maxit,oimg)

%u=SplitBregDebP(g,ker,blur,sigma,mu,delta,theta,frame,Level,wLevel)
%Split Bregman iteration for Deblurring with Preconditioning.
%The tight frame systems are framlets.
%Input:   g -- blurred image;
%       ker -- blurring kernel;
%      blur -- function handle of blurring of usage blur(img,ker).
%     sigma -- the sigma of noise
% mu, lambda -- mu and delta in linearized Bregman iteration.
%     theta -- regularized parameter theta used in preconditioner.
%     frame -- framelet system used. 0-Haar, 1-Piecewise Linear, 3-Picewise Cubic
%     Level -- framelet decomposition level.
%    wLevel -- weighting of thresholding on each level. If wLevel<=0, we 
%              use default values, the product of norms of filters. If 
%              wLevel>0, the thresholding is multiplied by wLevel from the 
%              current level to the next level. 

[D,R]=GenerateFrameletFilter(frame);
nD=length(D);
[m,n]=size(g);
cker=fliplr(flipud(ker));

BTg=blur(g,cker);

Transform=@(x)fft2(x);
InverseTransform=@(x)ifft2(x); 

%Compute the weighted thresholding parameters.
muLevel=getwThresh(mu,wLevel,Level,D);

%Compute the eigenvalues of P.
eigenP=eigenofP(ker,cker,lambda,m,n);  

normg=norm(g,'fro');

%initialization
u=zeros(m,n);
b=FraDecMultiLevel(u,D,Level);
d=b;
u=zeros(m,n);

nD=length(D);

for nstep=1:maxit 

    for innerstep=1:1
        % solve u.
        for ki=1:Level
            for ji=1:nD-1
                for jj=1:nD-1
                    C{ki}{ji,jj}=d{ki}{ji,jj}-b{ki}{ji,jj};
                end
            end
        end
        u=InverseTransform(Transform(lambda*FraRecMultiLevel(C,R,Level)+BTg)./eigenP);
    
        % solve d
        C=FraDecMultiLevel(u,D,Level);
        for ki=1:Level
            for ji=1:nD-1
                for jj=1:nD-1
                    d{ki}{ji,jj}=wthresh(C{ki}{ji,jj}+b{ki}{ji,jj},'s',muLevel{ki}{ji,jj}/lambda);
                end
            end
        end
    end

    error=0;
    for ki=1:Level
        for ji=1:nD-1
            for jj=1:nD-1
                if ((ji~=1)|(jj~=1))|(ki==Level)
                    deltab=C{ki}{ji,jj}-d{ki}{ji,jj};
                    error=error+norm(deltab,'fro')^2;
                    b{ki}{ji,jj}=b{ki}{ji,jj}+deltab;
                end
            end
        end
    end

    error=sqrt(error)/normg;
    disp(['error on step ' num2str(nstep) ' is ' num2str(error) ', and SNR is ' num2str(snr(u,oimg))]);
    if error<tol
        break;
    end
end

u(u>255)=255;u(u<0)=0;

function eigenP=eigenofP(ker,cker,lambda,m,n);  

[nker,mker]=size(ker);
tmp=zeros(m,n);tmp(1:nker,1:mker)=ker;
tmp=circshift(tmp,[-floor(nker/2),-floor(mker/2)]);
eigenP=abs(fft2(tmp)).^2+lambda;

%LD=length(D);
%LF=length(D{1});
%for l=1:Level
%    for i=1:LD-1
%        for j=1:LD-1
%          if (i~=1)|(j~=1)
%            nzs=2^(l-1)-1;
%            ILF=LF+nzs*(LF-1);
%            flt=zeros(ILF,ILF);
%            flt(1:nzs+1:ILF,1:nzs+1:ILF)=(D{i}')*D{j};
%            tmp=zeros(m,n);tmp(1:ILF,1:ILF)=flt;
%            tmp=circshift(tmp,[-floor(ILF/2),-floor(ILF/2)]);
%            eigenP=eigenP+lambda*(muLevel{l}{i,j})^2*abs(fft2(tmp)).^2;
%         end
%       end
%    end
%end    

function [D,R]=GenerateFrameletFilter(frame)
if frame==0          %Haar Wavelet
    D{1}=[0 1 1]/2;
    D{2}=[0 1 -1]/2;
    D{3}='cc';
    R{1}=[1 1 0]/2;
    R{2}=[-1 1 0]/2;
    R{3}='cc';
elseif frame==1      %Piecewise Linear Framelet
    D{1}=[1 2 1]/4;
    D{2}=[1 0 -1]/4*sqrt(2);
    D{3}=[-1 2 -1]/4;
    D{4}='ccc';
    R{1}=[1 2 1]/4;
    R{2}=[-1 0 1]/4*sqrt(2);
    R{3}=[-1 2 -1]/4;
    R{4}='ccc';
elseif frame==3      %Piecewise Cubic Framelet
    D{1}=[1 4 6 4 1]/16;
    D{2}=[1 2 0 -2 -1]/8;
    D{3}=[-1 0 2 0 -1]/16*sqrt(6);
    D{4}=[-1 2 0 -2 1]/8;
    D{5}=[1 -4 6 -4 1]/16;
    D{6}='ccccc';
    R{1}=[1 4 6 4 1]/16;
    R{2}=[-1 -2 0 2 1]/8;
    R{3}=[-1 0 2 0 -1]/16*sqrt(6);
    R{4}=[1 -2 0 2 -1]/8;
    R{5}=[1 -4 6 -4 1]/16;
    R{6}='ccccc';
end

function muLevel=getwThresh(mu,wLevel,Level,D);
nfilter=1;
nD=length(D);
if wLevel<=0
    for ki=1:Level
        for ji=1:nD-1
            for jj=1:nD-1
                muLevel{ki}{ji,jj}=mu*nfilter*norm(D{ji})*norm(D{jj});
            end
        end
        nfilter=nfilter*norm(D{1});
    end
else
    for ki=1:Level
        for ji=1:nD-1
            for jj=1:nD-1
                muLevel{ki}{ji,jj}=mu*nfilter;
            end
        end
        nfilter=nfilter*wLevel;
    end
end